
##Olivia Rodriguez 
#Zool 421
#Microbiome Tutorial

set.seed(2863)

###Step 1: Install packages###

if (!requireNamespace("BiocManager", quietly = TRUE))
  install.packages("BiocManager")

BiocManager::install("phyloseq")
BiocManager::install("microbiome")


#install the packages dplyr and ggplot2 below#

###Step 2: Load Packages###

library(phyloseq)
library(microbiome)
library(dplyr)
library(ggplot2)
library(ape)


###Step 3: Installing Q2 plugin###

if (!requireNamespace("devtools", quietly = TRUE)){install.packages("devtools")}
devtools::install_github("jbisanz/qiime2R") 

library(qiime2R)


###Step 4: Reading in data###

#Read in data table of sample reads#
SVs<-read_qza("feature_table.qza")

asv_table<-(SVs$data)

Asv_table<-data.frame(asv_table)

colnames(Asv_table) <- colnames(asv_table)


#Read in phylogenetic tree#
tree<- read_qza("rooted_tree.qza")

Tree<-tree$data

#Read in metadata (data table of information about samples)#

Metadata_final<-read.csv("moose-metadata.csv")

#Only do this step If you have a windows system.  You need to rename the SampleID column because R in windows renames the SampleID column when import. 
names(Metadata_final)[1]  <- "SampleID" 

# get column names to assure renaming worked 
colnames(Metadata_final)[1]

#Others without Windows systems, skip to this step
metadata_finalnames <- as.character(Metadata_final$SampleID)

rownames(Metadata_final) <- metadata_finalnames 


#Read in Taxonomy Table#
taxonomy<-read_qza("classified_rep_seqs.qza")
head(taxonomy$data)

#when taxonomy is imported, a single string is returned along with a confidence score.
#For many analysis we will want to break up this string and for that purpose the parse_taxonomy() function is provided#
taxonomy<-parse_taxonomy(taxonomy$data)
head(taxonomy)

taxonomy$Kingdom <- as.character(taxonomy$Kingdom)
taxonomy$Phylum <- as.character(taxonomy$Phylum)
taxonomy$Class <- as.character(taxonomy$Class)
taxonomy$Order <- as.character(taxonomy$Order)
taxonomy$Family <- as.character(taxonomy$Family)
taxonomy$Genus <- as.character(taxonomy$Genus)
taxonomy$Species <- as.character(taxonomy$Species)

taxonomy <- as.matrix(taxonomy)



###Step 5: Create phyloseq object###

phyloseq_16S_moose <- phyloseq(otu_table(Asv_table, taxa_are_rows = TRUE, errorIfNULL = TRUE), sample_data(Metadata_final), tax_table(taxonomy), phy_tree(Tree))

phyloseq_16S_moose


###Step 6: On your own###
#Skip this when running tutorial first time and start here when you get to step 4 in the Lab Microbiome Protocol.#
#(You will Subset the object to only include moose samples from a certain year. Example below for 2013)#
phyloseq_16S_2013 <- subset_samples(phyloseq_16S_moose, year == "2013") 


#: Create a sample metadata dataframe
meta <- data.frame(sample_data(phyloseq_16S_moose))


###Step 7: Diversity Analysis###

##Alpha diversity measurements##


#Calculate alpha diversity metrics then create a table containing those values
alpha <- data.frame(estimate_richness(phyloseq_16S_moose, split = TRUE, measures = NULL))

row.names(alpha) <- row.names(meta)

alpha$SampleID <- rownames(alpha) 
alpha_meta <- left_join(meta, alpha)


#Estimate Species Richness by year then by location#

plot_richness(phyloseq_16S_moose, x="location", measures = c("Chao1", "Shannon"), color = "year_text")

plot_richness(phyloseq_16S_moose, x="year_text", measures = c("Chao1", "Shannon"), color = "location")

#Plot a Shannon value boxplot for samples group by location and colored by year#
shannon_location <- 
  ggplot(alpha_meta, aes(location, Shannon, color = year_text)) +
  geom_boxplot() + labs(x = "Region") +
  geom_jitter(position = position_jitterdodge(jitter.width = 0)) +
  theme_bw()
shannon_location + theme(text=element_text(size=20))



##Beta diversity measures##

#Transform data: Needs to be compositional#

moose.comp <- microbiome::transform(phyloseq_16S_moose, "compositional")

#ordinate the data to PCoA design#
ord.pcoa.bray <- ordinate(moose.comp , "PCoA", "bray")

#plot the ordination with samples colored by year#
moose.bray <- plot_ordination(moose.comp, 
                              ord.pcoa.bray, color="location") 
moose.bray <- moose.bray + ggtitle("Bray-Curtis") + geom_point(size = 2)
moose.bray <- moose.bray + theme_classic() + scale_color_brewer("Region", palette = "Set2")
print(moose.bray + stat_ellipse())




###Step 8: Taxonomy Exploration###

#Subset moose dataset to just the observed Bacteria and remove any other microorganisms#
moose_bacteria <- subset_taxa(phyloseq_16S_moose, Kingdom=="Bacteria")

#Create an Initial basic plot of taxonomy grouped by location and identifying all bacterial families#
plot_bar(moose_bacteria, x="location", fill="Family") + geom_bar(aes(color=Family, fill=Family), stat="identity", position="stack")+ theme(legend.position="bottom")

#Sort the Families by abundance and pick the top 10 most abundant families#
top10F.names = sort(tapply(taxa_sums(moose_bacteria), tax_table(moose_bacteria)[, "Family"], sum), TRUE)[1:10]

#Cut down the physeq.tree data to only include the top 10 Families#
top10F = subset_taxa(moose_bacteria, Family %in% names(top10F.names))

#Plot top10 families
plot_bar(top10F, x="location", fill="Family", facet_grid = ~Family) + geom_bar(aes(color=Family, fill=Family), stat="identity", position="stack")+ theme(legend.position="bottom")


###Step 9: Optional##
#In order to create reproducible data and save all your work, you can export your objects#

#Export taxonomy#

tax<-as(tax_table(phyloseq_16S_moose),"matrix")
tax_cols <- colnames(tax)
tax<-as.data.frame(tax)
tax$taxonomy<-do.call(paste, c(tax[tax_cols], sep=";"))
for(co in tax_cols) tax[co]<-NULL
write.table(tax, "tax.txt", quote=FALSE, col.names=FALSE, sep="\t")

#export OTU table as biom file#


library(biomformat);packageVersion("biomformat")


otu<-as(otu_table(phyloseq_16S_moose),"matrix")
otu_biom<-make_biom(data=otu)
write_biom(otu_biom,"otu_biom.biom")

#Export metadata file

write.table(sample_data(phyloseq_16S_moose),"sample-metadata.txt", sep="\t", row.names=FALSE, col.names=TRUE, quote=FALSE)

#export tree file#
tree.seq = phy_tree(phyloseq_16S_moose)


write.tree(tree.seq,"phy_tree.tree")

#Save your entire phyloseq object#

saveRDS(phyloseq_16S_moose, "phyloseq_16S_moose.rds")

